package trace

import (
	"github.com/opentracing/opentracing-go"
	"github.com/opentracing/opentracing-go/ext"
	"github.com/opentracing/opentracing-go/log"

	"github.com/greenqloud/heimdall/pkg/logging"
)

type Span interface {
	opentracing.Span
}

type span struct {
	opentracing.Span
	logger    *logging.Logger
	hasHasErr bool
}

func (s *span) Finish() {
	s.logger.Debug("Finishing span")
	s.Span.Finish()
}

func (s *span) LogKV(alternatingKeyValues ...interface{}) {
	fields, err := log.InterleavedKVToFields(alternatingKeyValues...)
	if err != nil {
		s.LogFields(log.Error(err), log.String("function", "LogKV"))
		return
	}
	s.checkError(fields...)
	s.Span.LogKV(alternatingKeyValues...)
}

func (s *span) LogFields(fields ...log.Field) {
	s.checkError(fields...)
	s.Span.LogFields(fields...)
}

func (s *span) checkError(fields ...log.Field) {
	if s.hasHasErr {
		return
	}
	// catch standard opentracing keys and adjust to internal ones as per spec:
	// https://github.com/opentracing/specification/blob/master/semantic_conventions.md#log-fields-table
	for _, f := range fields {
		switch f.Key() {
		case "error", "err":
			ext.Error.Set(s, true)
			s.hasHasErr = true
		}
	}
}
